/* KallistiOS 1.1.8

   screenshot.c
   (c)2002 Dan Potter
 */

#include <stdio.h>
#include <stdlib.h>
#include <dc/video.h>
#include <kos/fs.h>


static unsigned int screens_counter=0;


CVSID("$Id: screenshot.c,v 1.1 2002/03/23 07:11:43 bardtx Exp $");

/*

Provides a very simple screen shot facility (dumps raw RGB PPM files from the
currently viewed framebuffer).

Destination file system must be writeable and have enough free space.

Assumes display is in 16-bit mode.

*/

int vid_screen_shot(global *g_point) {
	file_t	f;
	int	i, bsize;
	uint8	*buffer;
	uint16	pixel;
	uint8	r, g, b;
	char	header[256];

	char filename[512];

	strcpy(filename, g_point->base_dir);

	strcpy((filename + strlen(g_point->base_dir)), "screenshots/screen0000.ppm");

	int check=1;

int c=strlen(g_point->base_dir);

while (check)
	{
	filename[18+c]='0';
	filename[19+c]='0';
	filename[20+c]='0';
	filename[21+c]='0';

	screens_counter++;

	uint thousands, hundreds, tens, ones;

	thousands=screens_counter/1000;
	hundreds=(screens_counter-(1000*thousands))/100;
	tens=(screens_counter-(1000*thousands)-(100*hundreds))/10;
	ones=screens_counter-(1000*thousands)-(100*hundreds)-(10*tens);

	filename[18+c]+=thousands;
	filename[19+c]+=hundreds;
	filename[20+c]+=tens;
	filename[21+c]+=ones;

	check=fs_open((const char *)&filename, O_RDONLY);
	}

	fs_close(check);


	/* Allocate a new buffer so we can blast it all at once */
	bsize = vid_mode->width * vid_mode->height * 3;
	buffer = (uint8 *)malloc(bsize);
	if (buffer == NULL) {
		dbglog(DBG_ERROR, "\nvideo_screen_shot: can't allocate ss memory\n");
		return -1;
	}

	/* Write out each 16-bit pixel as 24 bits */
	for (i=0; i<vid_mode->width*vid_mode->height; i++) {
		pixel = vram_s[i];
		r = ((pixel >> 11) & 0x1f) << 3;
		g = ((pixel >> 5) & 0x3f) << 2;
		b = ((pixel >> 0) & 0x1f) << 3;
		buffer[i*3+0] = r;
		buffer[i*3+1] = g;
		buffer[i*3+2] = b;
	}

	/* Open output file */
	f = fs_open((const char *)&filename, O_WRONLY);
	if (!f) {
		printf("video_screen_shot: can't open output file %s", filename);
		free(buffer);
		return -1;
	}

	/* Write a small header */
	sprintf(header, "P6\n#KallistiOS Screen Shot\n%d %d\n255\n", vid_mode->width, vid_mode->height);
	if (fs_write(f, header, strlen(header)) != strlen(header)) {
		dbglog(DBG_ERROR, "video_screen_shot: can't write header to output file '%s'\n", (const char *)&filename);
		fs_close(f);
		free(buffer);
		return -1;
	}

	/* Write the data */
	if (fs_write(f, buffer, bsize) != (ssize_t)bsize) {
		dbglog(DBG_ERROR, "video_screen_shot: can't write data to output file '%s'\n",(const char *)&filename);
		fs_close(f);
		free(buffer);
		return -1;
	}

	fs_close(f);
	free(buffer);

	return 0;
}
